/// wizSDLJoystick.cpp
// Basic SDL Joystick Test program
#include "SDL.h"
//Also test SDL_mixer
#include "SDL_mixer.h"
//for executing the GP2X Wiz Menu on Exit
#include <unistd.h>

//screen dimensions
const int SCREEN_WIDTH=320;
const int SCREEN_HEIGHT=240;

//display surface
SDL_Surface* wizScreen = NULL;

//WIZ image
SDL_Surface* wizBMP = NULL;
//WIZ buttons
SDL_Surface* wizButtonBMP;

//event structure
SDL_Event event;

//joystick
SDL_Joystick* wizButtons;

//number of buttons
int numButtons;

//rectangles
SDL_Rect fillRect, wizRect, grabRect, drawRect;

//gameover trigger
bool gameover = false;

//music pointer
Mix_Music* backMusic;
//sound effects
//Mix_Chunk* sound1;
int volume = 64;

//main function
int main(int argc, char* argv[]) {
	//initialize SDL
	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK)==-1) {
		//error initializing SDL
		//report the error
		fprintf(stderr,"Could not initialize SDL!\n");
		//end the program
		exit(1);
	} else {
		//SDL initialized
		//report success
		fprintf(stdout,"SDL initialized properly!\n");
		//set up to uninitialize SDL at exit
		atexit(SDL_Quit);
	}

	//check for joysticks
	if(SDL_NumJoysticks()==0) {
		fprintf(stdout,"No joysticks!\n");
		exit(0);
	}

	//create windowed environment
	wizScreen = SDL_SetVideoMode(SCREEN_WIDTH,SCREEN_HEIGHT,0,SDL_ANYFORMAT);
	//error check
	if (wizScreen == NULL) {
		//report error
		fprintf(stderr,"Could not set up display surface!\n");
		//exit the program
		exit(1);
	}

	//load in Wiz BMP
	wizBMP=SDL_LoadBMP("wizSDLJoystickFiles/Wiz.bmp");
	wizButtonBMP=SDL_LoadBMP("wizSDLJoystickFiles/WizButtonPress.bmp");
	//error check
	if( (wizBMP == NULL) || (wizButtonBMP == NULL) ) {
		//report error
		fprintf(stderr,"Could not load bitmap surface!\n");
		//exit the program
		exit(1);
	}
	//set color keys
	SDL_SetColorKey(wizBMP,SDL_SRCCOLORKEY,SDL_MapRGB(wizBMP->format, 255, 0, 255));
	SDL_SetColorKey(wizButtonBMP,SDL_SRCCOLORKEY,SDL_MapRGB(wizButtonBMP->format, 255, 0, 255));
	
	//set width and height of wizRect to that of the BMP
	wizRect.w=wizBMP->w;
	wizRect.h=wizBMP->h;
	
	//start up sdl_mixer
	Mix_OpenAudio(MIX_DEFAULT_FREQUENCY,MIX_DEFAULT_FORMAT,1,4096);
	//load the chunks
	backMusic = Mix_LoadMUS("wizSDLJoystickFiles/back.wav");
	//allocate channels
	Mix_AllocateChannels(1);
	//play the chunk
	Mix_PlayMusic(backMusic,-1);
	
	//grab the first joystick
	wizButtons=SDL_JoystickOpen(0);

	//get the number of buttons
	numButtons=SDL_JoystickNumButtons(wizButtons);

	//enable joystick events
	SDL_JoystickEventState(SDL_ENABLE);

	//repeat until gameover triggered
	while(!gameover) {
		//wait for an event
		if(SDL_PollEvent(&event)==0) {
			//no event

			//clear out the entire screen
			fillRect.x=fillRect.y=0;
			fillRect.w=SCREEN_WIDTH;
			fillRect.h=SCREEN_HEIGHT;
			SDL_FillRect(wizScreen,&fillRect,SDL_MapRGB(wizScreen->format,0,0,50));

			//draw WIZ
			SDL_BlitSurface(wizBMP,NULL,wizScreen,&wizRect);

			//UP Press
			if( SDL_JoystickGetButton(wizButtons,0) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=0;
				grabRect.y=0;
				grabRect.w=drawRect.w=19;
				grabRect.h=drawRect.h=25;
				//set location on wizScreen
				drawRect.x=45;
				drawRect.y=66;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			//LEFT Press
			if( SDL_JoystickGetButton(wizButtons,2) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=18;
				grabRect.y=0;
				grabRect.w=drawRect.w=24;
				grabRect.h=drawRect.h=20;
				//set location on wizScreen
				drawRect.x=31;
				drawRect.y=80;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			//DOWN Press
			if( SDL_JoystickGetButton(wizButtons,4) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=41;
				grabRect.y=0;
				grabRect.w=drawRect.w=19;
				grabRect.h=drawRect.h=24;
				//set location on wizScreen
				drawRect.x=46;
				drawRect.y=90;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			//RIGHT Press
			if( SDL_JoystickGetButton(wizButtons,6) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=59;
				grabRect.y=0;
				grabRect.w=drawRect.w=25;
				grabRect.h=drawRect.h=18;
				//set location on wizScreen
				drawRect.x=54;
				drawRect.y=80;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			
			//MENU Press
			if( SDL_JoystickGetButton(wizButtons,8) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=83;
				grabRect.y=0;
				grabRect.w=drawRect.w=19;
				grabRect.h=drawRect.h=20;
				//set location on wizScreen
				drawRect.x=45;
				drawRect.y=123;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			//SELECT Press
			if( SDL_JoystickGetButton(wizButtons,9) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=101;
				grabRect.y=0;
				grabRect.w=drawRect.w=20;
				grabRect.h=drawRect.h=18;
				//set location on wizScreen
				drawRect.x=257;
				drawRect.y=126;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			
			//L Press
			if( SDL_JoystickGetButton(wizButtons,10) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=0;
				grabRect.y=24;
				grabRect.w=drawRect.w=35;
				grabRect.h=drawRect.h=35;
				//set location on wizScreen
				drawRect.x=47;
				drawRect.y=12;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			//R Press
			if( SDL_JoystickGetButton(wizButtons, 11) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=34;
				grabRect.y=24;
				grabRect.w=drawRect.w=35;
				grabRect.h=drawRect.h=35;
				//set location on wizScreen
				drawRect.x=239;
				drawRect.y=12;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			
			//A Press
			if( SDL_JoystickGetButton(wizButtons,12) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=120;
				grabRect.y=0;
				grabRect.w=drawRect.w=23;
				grabRect.h=drawRect.h=20;
				//set location on wizScreen
				drawRect.x=242;
				drawRect.y=80;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			//B Press
			if( SDL_JoystickGetButton(wizButtons,13) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=142;
				grabRect.y=0;
				grabRect.w=drawRect.w=20;
				grabRect.h=drawRect.h=18;
				//set location on wizScreen
				drawRect.x=270;
				drawRect.y=82;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			//X Press
			if( SDL_JoystickGetButton(wizButtons,14) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=161;
				grabRect.y=0;
				grabRect.w=drawRect.w=19;
				grabRect.h=drawRect.h=21;
				//set location on wizScreen
				drawRect.x=258;
				drawRect.y=92;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			//Y Press
			if( SDL_JoystickGetButton(wizButtons,15) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=178;
				grabRect.y=0;
				grabRect.w=drawRect.w=19;
				grabRect.h=drawRect.h=23;
				//set location on wizScreen
				drawRect.x=257;
				drawRect.y=67;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
			}
			
			//Vol+ Press
			if( SDL_JoystickGetButton(wizButtons,16) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=68;
				grabRect.y=19;
				grabRect.w=drawRect.w=25;
				grabRect.h=drawRect.h=23;
				//set location on wizScreen
				drawRect.x=87;
				drawRect.y=185;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
				//increase volume
				volume++;
			}
			//Vol- Press
			if( SDL_JoystickGetButton(wizButtons,17) ) {
				//grab correct button pressed graphic from BMP
				grabRect.x=90;
				grabRect.y=19;
				grabRect.w=drawRect.w=25;
				grabRect.h=drawRect.h=23;
				//set location on wizScreen
				drawRect.x=60;
				drawRect.y=185;
				//Show Button Press
				SDL_BlitSurface(wizButtonBMP,&grabRect,wizScreen,&drawRect);
				//decrease volume
				volume--;
			}
			
			//update the screen
			SDL_UpdateRect(wizScreen,0,0,0,0);
			
			//set music volume
			Mix_VolumeMusic(volume);
			
			//set gameover on Vol+ & Vol- press
			if( SDL_JoystickGetButton(wizButtons,16) && SDL_JoystickGetButton(wizButtons,17) )
				gameover = true;
		} else {
			//event occurred, check for quit
			if(event.type==SDL_QUIT) break;
		}
	}

	//close the joystick
	SDL_JoystickClose(wizButtons);
	
	//CLEANUP SDL_Mixer
	//deallocate channels
	Mix_AllocateChannels(0);
	//free the chunks
	Mix_FreeMusic(backMusic);
	//close sdl_mixer
	Mix_CloseAudio();
	
	//normal termination
	fprintf(stdout,"Terminating normally.\n");


	//Return to Wiz menu
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
	
	//return 0
	return(0);
} 
